<?php

const AMQPLIB_HOME_DIR = '/home/mgportal/';
define('vendor_dir', AMQPLIB_HOME_DIR . "vendor/");

const AMQPLIB_VENDOR_DIR = vendor_dir;

require_once(AMQPLIB_VENDOR_DIR . 'autoload.php');


use PhpAmqpLib\Channel\AMQPChannel;
use PhpAmqpLib\Connection\AMQPSocketConnection;
use PhpAmqpLib\Connection\AMQPStreamConnection;
use PhpAmqpLib\Message\AMQPMessage;
use PhpAmqpLib\Connection\AMQPSSLConnection;

class AMQP extends AMQPBase {

    public function __construct($host = null, $user = null, $asset = null) {
        parent::__construct($host, $user, $asset);
        if ($host != null) {
            $this->setParameters($host, $user);
        }
        $this->wait_timeout = null;         // timeout on the wait loop.
    }

    public function setPort($port) {
        $this->port = $port;
    }

    public function setParameters($host, $user) {
        parent::setParameters($host, $user);
    }

    public function connect($useSSL = false) {
        global $Log;
        $this->useSSL = $useSSL;
        if ($useSSL) {
            // If the port was left as the default, use the default SSL port.
            if ($this->port == AMQPBase::PORT) {
                $this->port = AMQPBase::SSL_PORT;
            }
            $ssl_options = $this->setSSLOptions(AMQPBase::CERTS_BASE_DIR);
            $options = array();
            if ($this->heartbeat > 0) {
                $options['heartbeat'] = $this->heartbeat;
            }
            if ($this->timeout != null) {
                $Log->logInfo('setting read_write_timeout: ' . $this->timeout);
                $options['read_write_timeout'] = $this->timeout;
            }
            $this->connection = new PhpAmqpLib\Connection\AMQPSSLConnection($this->host,
                                                                            $this->port,
                                                                            $this->user['name'],
                                                                            $this->user['password'],
                                                                            $this->vHost,
                                                                            $ssl_options,
                                                                            $options);
        } else {
            $timeout = 3.0;
            $heartbeat = $this->heartbeat;
            $keepalive = false;
            // set timeout
            if ($this->timeout !== null) {
                $timeout = $this->timeout;
                $Log->logInfo('setting read_write_timeout: ' .$timeout);
            }
            $this->connection = new PhpAmqpLib\Connection\AMQPStreamConnection($this->host,
                                                                                $this->port,
                                                                                $this->user['name'],
                                                                                $this->user['password'],
                                                                                $this->vHost,
                                                                                false,                  // insist
                                                                                AMQPBase::LOGIN_METHOD, // login method
                                                                                null,                   // login response
                                                                                AMQPBase::LOCALE,       // locale
                                                                                3.0,                    // connection timeout
                                                                                $timeout,               // read write timeout
                                                                                null,                   // context
                                                                                $keepalive,             // keepalive
                                                                                $heartbeat);            // heartbeat
        }
    }

    public function createChannel() {
        $this->channel = $this->connection->channel();
        return $this->getChannel();
    }

    public function createExchange() {
        // exchange_declare
        //  exchange name
        //  type - e.g., direct, fanout, topic, headers
        //  passive (active if false)
        //  durable
        //  auto_delete
        //  internal (false default)
        //  nowait (if false, wait, false default)
        //  arguments (null default)
        //  ticket (null deafult)
        $this->channel->exchange_declare('cloud-default', 'direct', false, true, false);
        $this->exchange = 'cloud-default';
        return $this->getExchange();
    }

    public function consume($consumerFunction, $queueName = null) {
        $queue = $queueName != null ? $queueName : $this->queue;
        // basic_consume
        //  queue name
        //  consumer tag
        //  no ack (false, acks turned on)
        //  exclusive - queues only with current connection
        //  no wait (false, wait).
        $this->channel->basic_consume($queue, $queue, false, false, false, false, $consumerFunction);
        register_shutdown_function(array($this, 'closeConnection'));
    }

    public function createQueue($routing_key, $auto_delete = true) {
        // queue_declare
        //  queue name
        //  passive
        //  durable
        //  exclusive
        //  auto-delete
        list($routing_key,,) = $this->channel->queue_declare($routing_key, false, false, false, $auto_delete);
        $this->queue = $routing_key;
        $this->bindQueue($routing_key, 'cloud-default');
        return $this->getQueue();
    }

    public function bindQueue($queueKey, $exchangeKey){
        $this->channel->queue_bind($queueKey, $exchangeKey);
    }

    public function publish($message, $exchangeName, $queueName, $options){
        $amqpMessage = new PhpAmqpLib\Message\AMQPMessage($message, $options);
        // basic publish
        //  exchange
        //  queue name (routing key)
        $this->channel->basic_publish($amqpMessage, $exchangeName, $queueName);
    }

    public function ack($msg) {
        // acknowledge message
        $msg->delivery_info['channel']->basic_ack($msg->delivery_info['delivery_tag']);
    }

    public function respond($msg, $encoded_message) {
        $newMessage = new PhpAmqpLib\Message\AMQPMessage($encoded_message); //, array('correlation_id' => $originalMessage->get('correlation_id')));
        $msg->delivery_info['channel']->basic_publish($newMessage, '', $msg->get('reply_to'));
    }

    public function waitLoop() {
        while(count($this->channel->callbacks)) {
            $allowed_methods = null;
            $non_blocking = false;
            $waitTimeout = 0;
            // lmc - TODO
            // This is a timing issue with SSL on the server.  Will hang!
            if (!$this->useSSL && $this->wait_timeout !== null) {
                $waitTimeout = $this->wait_timeout;
            }
            $this->channel->wait($allowed_methods, $non_blocking, $waitTimeout);
        }
    }

    public function disconnect() {
        $this->closeConnection();
    }

    public function closeConnection(){
        if (is_object($this->channel)) {
            $this->channel->close();
        }
        if (is_object($this->connection)) {
            $this->connection->close();
        }
    }

    public function reconnect() {
        $this->connection->reconnect();
    }

    public function setWaitTimeout($timeout) {
        $this->wait_timeout = $timeout;
    }
}

?>
