<?php

class Configuration {
    const CLIENT = "client";
    const SERVER = "server";
    const LOCALHOST = "localhost";
    const CONFIG = "mgportal.conf";
    
    // Capabilities
    const MONITOR = "monitor";
    const MANAGE = "manage";
    const NONE = "none";
    
    // Ports
    const PORT = 5672;
    const SSL_PORT = 5671;

    // AMQP timeout default
    const DEFAULT_TIMEOUT = 0;
    const DEFAULT_HEARTBEAT = 0;    // not set

    // Default days of data to send to network manager
    const DEFAULT_DATA_DAYS = 7;

    // Default number of open ports
    const DEFAULT_PORT_COUNT = 5;
    
    // Command List
    const CMD_PROTECT_INFO = "protection_information";
    const CMD_PROTECTED_MONTHLY = "monthly";
    const CMD_DEVICE_CHECK = "device";
    const CMD_GET_JOBS = "get-jobs";
    const CMD_GET_ALERTS = "get-alerts";
    const CMD_GET_BACKUPS = "get-backups";
    const CMD_GET_HOT_COPIES = "get-hot-copies";
    const CMD_GET_ALL = "get-all";
    const CMD_GET_KEY = "get-key";
    const CMD_ADD_KEY = "add-key";
    const CMD_OPEN_TUNNEL = "open-tunnel";
    const CMD_CLOSE_TUNNEL = "close-tunnel";
    const CMD_CHECK_TUNNEL = "check-tunnel";
    const CMD_GET_CONNECTION_STATUS = "get-connection-status";
    
    // Default Vendor
    const DEFAULT_VENDOR = "unitrends";
    const DEFAULT_API = "http://localhost/api/";
    
    // Route for AMQP messages for monitoring (if not using REST)
    const DEVICE_MONITORING_ROUTE = 'appliance_to_cloud';

    // If allowance is this, allow devices that are already in the portal table to update
    const ALLOW_EXISTING_DEVICES = "existing";

    // AMQP password
    const OBFUSCATED_PW = '(not visible)';
    
    function __construct($BaseDir, $configFile = Configuration::CONFIG, $type) {
        $this->configFile = $BaseDir . '/' . $configFile;
        $this->type = $type == "client" ? Configuration::CLIENT : Configuration::SERVER;
        $this->host = Configuration::LOCALHOST;
        $this->username = "";
        $this->password = "";
        $this->port = Configuration::PORT;
        $this->vendor = Configuration::DEFAULT_VENDOR;
        $this->vendor_local_API = Configuration::DEFAULT_API;
        $this->log_level = Logger::NORMAL;
        $this->log_file_count = Logger::LOG_FILE_COUNT;
        $this->log_file_size = Logger::LOG_FILE_SIZE;
        $this->activity = Configuration::MONITOR;  // one of "monitor", "manage", or "none"
        $hostObject = array(   'host' => $this->host,
            'connection_username' => $this->username,
            'connection_password' => $this->password,
            'activity' => $this->activity);
        $this->hosts = array($hostObject);

        $this->allowUnsupportedDevice = false;

        $this->useAMQPMonitoring = false;
        $this->AMQP_read_timeout = Configuration::DEFAULT_TIMEOUT;
        $this->AMQP_heartbeat = Configuration::DEFAULT_HEARTBEAT;
        $this->AMQP_wait_timeout = null;        // amount of time to wait on the server for commands to the client.

        $this->dataDays = Configuration::DEFAULT_DATA_DAYS;

        $this->portCount = Configuration::DEFAULT_PORT_COUNT;

        $this->AMQPLib = false;
        $this->useSSL = false;
    }
    
    function load_configuration() {
        global $Log;
        $result = true;
        if (($jsonConfig = $this->parse_config_file()) === false) {
            return false;
        }
        if ($this->type == Configuration::CLIENT) {
            if (isset($jsonConfig['client'])) {
                $client = $jsonConfig['client'];
                if (isset($client['monitor_server'])) {
                    $this->host = $client['monitor_server'];
                    if (isset($client['connection_username'])) {
                        $this->username = $client['connection_username'];
                    }
                    if (isset($client['connection_password'])) {
                        $this->password = $client['connection_password'];
                    }
                    if (isset($client['activity'])) {
                        $activity = strtolower($client['activity']);
                        if ($activity == Configuration::NONE || $activity == Configuration::MANAGE || $activity == Configuration::MONITOR) {
                            $this->activity = $activity;
                        } else {
                            $Log->logError("Configuration error: only activity of 'non', 'manage', or 'monitor' are supported, will fall back to 'monitor'");                         
                        }
                    }
                    $hostObject = array(   'host' => $this->host,
                        'connection_username' => $this->username,
                        'connection_password' => $this->password,
                        'activity' => $this->activity);
                    $this->hosts = array($hostObject);
                }
                if (isset($client['monitor_servers'])) {
                    if (is_string($client['monitor_servers'])) {
                        $monitor_servers = $this->buildHosts($client['monitor_servers']);
                    } else {
                        $monitor_servers = $client['monitor_servers'];
                    }
                    $this->hosts = $monitor_servers;
                    $this->host = null;
                }
                if (isset($client['vendor']) && isset($client['vendor_local_api'])) {
                    $this->setVendor($client['vendor'], $client['vendor_local_api']);
                }
                if (isset($client['log_level'])) {
                    $this->setLogLevel($client['log_level']);
                }
                if (isset($client['log_file_count'])) {
                    $this->setLogFileCount($client['log_file_count']);
                }
                if (isset($client['log_file_size'])) {
                    $this->setLogFileSize($client['log_file_size']);
                }
                if (isset($client['port'])) {
                    $this->setPort($client['port']);
                }
                if (isset($client['useSSL'])) {
                    $this->setSSL($client['useSSL']);
                }
                if (isset($client['AMQPLib'])) {
                    $this->setAMQPLib($client['AMQPLib']);
                }
                if (isset($client['amqp_monitoring'])) {
                    $this->setAMQPMonitoring($client['amqp_monitoring']);
                }
                if ($this->host === null && $this->hosts === null) {
                    $Log->logError("Configuration error: no monitor_server for client");
                    $result = false;
                }
                if (isset($client['data_days'])) {
                    $this->setDays($client['data_days']);
                }
                if (isset($client['timeout'])) {
                    $this->setTimeout($client['timeout']);
                }
                if (isset($client['heartbeat'])) {
                    $this->setHeartbeat($client['heartbeat']);
                }
            }
        } else if ($this->type == Configuration::SERVER) {
            if (isset($jsonConfig['server'])) {
                $server = $jsonConfig['server'];
                if (isset($server['monitor_server'])) {
                    $this->host = $server['monitor_server'];
                    if (isset($server['connection_username'])) {
                        $this->username = $server['connection_username'];
                    }
                    if (isset($server['connection_password'])) {
                        $this->password = $server['connection_password'];
                    }
                    if (isset($server['activity'])) {
                        $activity = strtolower($server['activity']);
                        if ($activity == Configuration::NONE || $activity == Configuration::MANAGE || $activity == Configuration::MONITOR) {
                            $this->activity = $activity;
                        }
                    }
                    $hostObject = array(   'host' => $this->host,
                        'connection_username' => $this->username,
                        'connection_password' => $this->password,
                        'activity' => $this->activity);
                    $this->hosts = array($hostObject);
                }
                if (isset($server['monitor_servers'])) {
                    if (is_string($server['monitor_servers'])) {
                        $monitor_servers = $this->buildHosts($server['monitor_servers']);
                    } else {
                        $monitor_servers = $server['monitor_servers'];
                    }
                    $this->hosts = $monitor_servers;
                    $this->host = null;
                }
                if (isset($server['vendor']) && isset($server['vendor_local_api'])) {
                    $this->setVendor($server['vendor'], $server['vendor_local_api']);
                }
                if (isset($server['log_level'])) {
                    $this->setLogLevel($server['log_level']);
                }
                if (isset($server['log_file_count'])) {
                    $this->setLogFileCount($server['log_file_count']);
                }
                if (isset($server['log_file_size'])) {
                    $this->setLogFileSize($server['log_file_size']);
                }
                if (isset($server['port'])) {
                    $this->setPort($server['port']);
                }
                if (isset($server['useSSL'])) {
                    $this->setSSL($server['useSSL']);
                }
                if (isset($server['AMQPLib'])) {
                    $this->setAMQPLib($server['AMQPLib']);
                }
                if (isset($server['allow_unsupported_devices'])) {
                    $this->setDeviceAllowance($server['allow_unsupported_devices']);
                }
                if ($this->host === null && $this->hosts === null) {
                    global $Log;
                    $Log->logError("Configuration error: no monitor_server for server");
                    $result = false;
                }
                if (isset($server['timeout'])) {
                    $this->setTimeout($server['timeout']);
                }
                if (isset($server['heartbeat'])) {
                    $this->setHeartbeat($server['heartbeat']);
                }
                if (isset($server['wait_timeout'])) {
                    $this->setWaitTimeout($server['wait_timeout']);
                }
                if (isset($server['port_count'])) {
                    $this->setPortCount($server['port_count']);
                }
            }
        }
        return $result;
    }

    /*
     * Converts a host string definition into an array
     * String format:
     * "property_1_host_1:value_1_host_1,property_2_host_1:value_2_host_1;property_1_host_2:value_1_host_2"
     */
    private function buildHosts($hostsString) {
        $result = array();
        $hostArray = explode(';', $hostsString);
        foreach ($hostArray as $hostString) {
            $propArray = explode(',', $hostString);
            $properties = array();
            foreach ($propArray as $propString) {
                $property = explode('^', $propString);
                $properties[$property[0]] = $property[1];
            }
            $result[] = $properties;
        }
        //global $Log;
        //$Log->logInfo("converted hosts " . $hostsString);
        //var_dump($result);
        return $result;
    }
    
    /*
     * This function loads the configuration file into a PHP array, returning the array on success or false on failure.
     * Logs an error message if the file is missing or the contents cannot be decoded into valid JSON.
     */
    private function parse_config_file() {
        global $Log;
        $jsonConfig = false;
        $config = @file_get_contents($this->configFile);
        if ($config !== false) {
            $jsonConfig = json_decode($config, true);
            if ($jsonConfig === null) {
                $message = $this->json_last_error_msg();
                $Log->logError("Error loading configuration file: " . $message);
                $jsonConfig = false;
            }
        } else {
            $Log->logError("Error opening config file; will use default configuration settings and monitor server " . $this->host);
        }
        return $jsonConfig;
    }
    
    /*
     * This function sanity-checks and set the vendor and vendor API prefix if set.
     */
    private function setVendor($vendor, $vendor_api) {
        global $Log;
        if (strpos($vendor, ' ') !== false) {
            $Log->logError("Configuration error: vendor name cannot contain spaces.");
        } else {
            $this->vendor = $vendor;
        }
        if (substr($vendor_api, -1) !== '/') {
            $Log->logError("Configuration error: vendor local API prefix must end in '/', adding it.");
            $vendor_api .= '/';
        }
        if (substr($vendor_api, 0, 4) !== "http") {
            $Log->logError("Configuration error: vendor local API prefix must start with http or https, adding http.");
            $vendor_api = 'http://' . $vendor_api;
        }
        $this->vendor_local_API = $vendor_api;       
    }
    
    /*
     * This function sanity checks and sets the log level.
     */
    private function setLogLevel($log_level) {
        global $Log;
        if (!is_numeric($log_level)) {
            $Log->logError("Configuration error: log_level must be numeric.");
        } else {
            $this->log_level = $log_level;
        }
    }

    /*
     * This function sanity checks and sets the # of log files.
     */
    private function setLogFileCount($log_files) {
        global $Log;
        if (!is_numeric($log_files)) {
            $Log->logError("Configuration error: log_file_count must be numeric.");
        } else if ($log_files < 0 || $log_files > 9) {
            $Log->logError("Configuration error: log_file_count must be between 1 and 9.");
        } else {
            $this->log_file_count = $log_files;
        }
    }

    /*
     * This function sanity checks and sets the log file size in MB.
     */
    private function setLogFileSize($log_file_size) {
        global $Log;
        if (!is_numeric($log_file_size)) {
            $Log->logError("Configuration error: log_file_size must be numeric.");
        } else if ($log_file_size < 0 || $log_file_size > 1024) {
            $Log->logError("Configuration error: log_file_size must be between 1 and 1024 (MB).");
        } else {
            $this->log_file_size = $log_file_size;
        }
    }
    
    /*
     * This function sanity checks and sets the port.
     */    
    private function setPort($port) {
        global $Log;
        if (!is_numeric($port)) {
            $Log->logError("Configuration error: port must be numeric, using default port.");
        } else {
            $this->port = $port;
        }
    }

    /*
     * This function sets usage of SSL.
     * */
    private function setSSL($useSSL) {
        global $Log;
        if (!is_bool($useSSL)) {
            $Log->logError("Configuration error: useSSL must be true or false");
        } else {
            $this->useSSL = $useSSL;
        }
    }

    /*
    * This function sanity checks and sets the port.
    */
    private function setDeviceAllowance($allowance) {
        global $Log;
        if (!is_bool($allowance)) {
            if ($allowance == Configuration::ALLOW_EXISTING_DEVICES) {
                // See if the device is already existing in the devices table.  Allow if it is.
                $this->allowUnsupportedDevice = Configuration::ALLOW_EXISTING_DEVICES;
            } else {
                $Log->logError("Configuration error: supported device allowance must be either true, false or 'existing'.");
            }
        } else {
            $this->allowUnsupportedDevice = $allowance;
        }
    }
    
    public function getDeviceAllowance() {
        return $this->allowUnsupportedDevice;
    }
    
    public function getHost() {
        return $this->host;
    }

    public function getHosts() {
        return $this->hosts;
    }
    
    /*
     * This function loops through the hosts and returns the first manager (array).  If no managers are found returns null.
     */
    public function getFirstManager() {
        $manager = null;
        foreach ($this->hosts as $entry) {
            $host = $entry['host'];
            $isManaged = $this->isManagedByHost($host);
            if ($isManaged) {
                $manager = $entry;
                break;
            }
        }
        return $manager;
    }
    
    public function getUser($host, $fallbackUser = "") {
        $user = false;
        $userFound = false;
        foreach ($this->hosts as $entry) {
            if ($entry['host'] == $host) {
                if (isset($entry['connection_username']) && $entry['connection_username'] != "") {
                    $user = array('name' => $entry['connection_username'], 'password' => $entry['connection_password']);
                    $userFound = true;
                }
                break;
            }
        }
        if (!$userFound && $fallbackUser != "") {
            $user = array('name' => $fallbackUser, 'password' => Configuration::OBFUSCATED_PW);
        }
        return $user;
    }
    
    public function getVendor() {
        return $this->vendor;
    }
    
    public function get_local_url() {
        return $this->vendor_local_API;
    }
    
    public function getLogLevel() {
        return $this->log_level;
    }

    public function getLogFileCount() {
        return $this->log_file_count;
    }

    public function getLogFileSize() {
        return $this->log_file_size;
    }

    public function getPort() {
        return $this->port;
    }
    
    public function anyDeviceManagement() {
        $managed = false;
        foreach ($this->hosts as $entry) {
            if ($entry['activity'] == Configuration::MANAGE) {
                $managed = true;
                break;
            }
        }    
        return $managed;
    }

    public function anyDeviceMonitoring() {
        $monitored = false;
        foreach ($this->hosts as $entry) {
            if ($entry['activity'] == Configuration::MANAGE || $entry['activity'] == Configuration::MONITOR) {
                $monitored = true;
                break;
            }
        }
        return $monitored;
    }
    
    public function isManagedByHost($host) {
        $managed = false;
        foreach ($this->hosts as $entry) {
            if ($entry['host'] == $host) {
                $managed = isset($entry['activity']) && $entry['activity'] == Configuration::MANAGE;
                break;
            }
        }
        return $managed;
    }

    public function isMonitoredByHost($host) {
        $monitored = false;
        foreach ($this->hosts as $entry) {
            if ($entry['host'] == $host) {
                $monitored = isset($entry['activity']) && ($entry['activity'] == Configuration::MANAGE || $entry['activity'] == Configuration::MONITOR);
                break;
            }
        }
        return $monitored;
    }
    
    public function get_command($cmd_arg) {
        $command = false;
        switch($cmd_arg) {
            case "protection":
            case "protect":
            case "protected":
                $command = Configuration::CMD_PROTECT_INFO;
                break;
            case "monthly":
                $command = Configuration::CMD_PROTECTED_MONTHLY;
                break;
            case "device-check":
            case "device":
                $command = Configuration::CMD_DEVICE_CHECK;
                break;
            case "get-jobs":
            case "jobs":
                $command = Configuration::CMD_GET_JOBS;
                break;
            case "get-alerts":
            case "alerts":
                $command = Configuration::CMD_GET_ALERTS;
                break;
            case "get-backups":
            case "backups":
                $command = Configuration::CMD_GET_BACKUPS;
                break;
            case "hot-copies":
            case "copies":
            case "replication":
                $command = Configuration::CMD_GET_HOT_COPIES;
                break;
            case "all":
            case "get-all":
                $command = Configuration::CMD_GET_ALL;
                break;
            case "getkey":
                $command = Configuration::CMD_GET_KEY;
                break;
            case "addkey":
                $command = Configuration::CMD_ADD_KEY;
                break;
            case self::CMD_GET_CONNECTION_STATUS:
                $command = self::CMD_GET_CONNECTION_STATUS;
                break;
            default:
                $command = false;
                break;
        }
        return $command;
    }
    
    public function add_message_header(&$message, $command) {
        // Set meta command and timestamp.
        $message["meta_command"] = $command;
        $message["meta_timestamp"] = time();       
    }
    
    public function generate_route($from, $to) {
        return 'route.' . $from . '.to.' . $to;
    }
    
    public function generate_route_reply($queue) {
        return $queue . '.reply-to';
    }
    
    /*
     * This function will be in PHP 5.5, but for now, use this version.
     */
    private function json_last_error_msg() {
        if (!function_exists('json_last_error_msg')) {
            static $JSON_ERRORS = array ();
            $JSON_ERRORS[JSON_ERROR_NONE] = "No error";
            $JSON_ERRORS[JSON_ERROR_DEPTH] = "maximum level of configuration definitions nesting exceeded";
            $JSON_ERRORS[JSON_ERROR_STATE_MISMATCH] = "bad JSON format";
            $JSON_ERRORS[JSON_ERROR_CTRL_CHAR] = "control characters detected";
            $JSON_ERRORS[JSON_ERROR_SYNTAX] = $JSON_ERRORS[JSON_ERROR_STATE_MISMATCH];
            $JSON_ERRORS[JSON_ERROR_UTF8] = "mal-formed UTF-8 characters detected";
            if (defined("JSON_ERROR_RECURSION") && defined("JSON_ERORR_INF_OR_NAN") && defined("JSON_ERROR_UNSUPPORTED_TYPE")) {
                $JSON_ERRORS[JSON_ERROR_RECURSION] = "recursive references detected";
                $JSON_ERRORS[JSON_ERROR_INF_OR_NAN] = "bad numeric values detected";
                $JSON_ERRORS[JSON_ERROR_UNSUPPORTED_TYPE] = "bad property type detected";               
            }
            $errno = json_last_error();
            $errorMessage = isset($JSON_ERRORS[$errno]) ? $JSON_ERRORS[$errno] : "unknown error";
        } else {
            $errorMessage = json_last_error_msg();
        }
        $errorMessage .= "; will use default configuration settings with host " . $this->host;
        return $errorMessage;
    }
    
    private function setAMQPMonitoring($use_AMQP) {
        global $Log;
        if (!is_bool($use_AMQP)) {
            $Log->logError("Configuration error: using AMQP for monitoring must be true or false only.");
        } else {
            $this->useAMQPMonitoring = $use_AMQP;
        }       
    }

    private function setAMQPLib($useAMQPLib) {
        global $Log;
        if (!is_bool($useAMQPLib)) {
            $Log->logError("Configuration error: using AMQP Lib must be true or false only.");
        } else {
            $this->AMQPLib = $useAMQPLib;
        }
    }
    
    public function getAMQPMonitoring() {
        return $this->useAMQPMonitoring;
    }


    private function setTimeout($timeout) {
        $this->AMQP_read_timeout = $timeout;
    }

    public function getTimeout() {
        return $this->AMQP_read_timeout;
    }

    private function setWaitTimeout($timeout) {
        $this->AMQP_wait_timeout = $timeout;
    }

    public function getWaitTimeout() {
        return $this->AMQP_wait_timeout;
    }

    private function setHeartbeat($heartbeat) {
        $this->AMQP_heartbeat = $heartbeat;
    }

    public function getHeartbeat() {
        return $this->AMQP_heartbeat;
    }

    private function setPortCount($count) {
        $this->portCount = $count;
    }

    public function getPortCount() {
        return $this->portCount;
    }

    /*
    * This function sets the number of days to fetch data (on commands where applicable).
    */
    private function setDays($days) {
        global $Log;
        if (!is_numeric($days)) {
            $Log->logError("Configuration error: data_days must be numeric.");
        } else {
            $this->dataDays = $days;
        }
    }

    public function getDays() {
        return $this->dataDays;
    }

    public function useAMQPLib() {
        return $this->AMQPLib;
    }

    public function useSSL() {
        return $this->useSSL;
    }
}
?>
