#!/usr/bin/php
<?php

const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const VENDOR_DIR = "vendor/";
const CONFIG_FILE = "mgportal.conf";

require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'REST.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');
require_once(BASE_DIR . BIN_DIR . 'AMQP-base.php');

$Log = new Logger();
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "client");
$Config->load_configuration();
if ($Config->useAMQPLib()) {
    require_once(BASE_DIR . BIN_DIR . 'AMQPlib.php');
} else {
    require_once(BASE_DIR . BIN_DIR . 'AMQP.php');
}
$REST = new REST($Config);
$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile('from_cloud');
$Log->setLogFileCount($Config->getLogFileCount());
$Log->setLogFileSize($Config->getLogFileSize());

/*
 * Check to see if monitoring is turned off and exit if so.
 */
if (!$Config->anyDeviceMonitoring()) {
    $Log->logInfo("This device is not being managed or monitored, exiting.");

    exit(-1);
} else if ($Config->anyDeviceManagement()) {
    // check rpms
    $output = shell_exec("rpm -q librabbitmq php-pecl-amqp");
    if (strstr($output, "librabbitmq") && strstr($output, "php-pecl-amqp")) {
        // all okay, packages present.
    } else {
        $Log->logError("Management cannot be performed until dependent packages, librabbitmq and php-amqp, are installed, exiting");
        exit(-1);
    }
}

$vendor = null;
$vendorName = $Config->getVendor();
if ($vendorName !== null) {
    require_once(BASE_DIR . BIN_DIR . VENDOR_DIR . $vendorName . '.php');
    $vendor = new Vendor($REST, $Config);
}
if ($vendor === null) {
    $Log->logError("Vendor information not found; please check config file for 'vendor'");
    exit(-1);
}

/*
 * Get the asset tag for routing key.
 */
$license = $REST->run('license');
if ($license === false || !isset($license['asset_tag'])) {
    $Log->logError("Error getting device asset");
    exit(-1);
}
$asset = $license['asset_tag'];

if ($asset !== false) {
    $manager = $Config->getFirstManager();
    
    if ($manager !== null) {
        try {
            $routing_key = $Config->generate_route('cloud', $asset);
            
            $AMQP = new AMQP($manager['host'], $Config->getUser($manager['host'], $asset), $asset);
            $AMQP->setPort($Config->getPort());
            $AMQP->setHeartbeat($Config->getHeartbeat());
            $AMQP->setTimeout($Config->getTimeout());
            $AMQP->connect($Config->useSSL());
            $channel = $AMQP->createChannel();
            $exchange = $AMQP->createExchange();

            if ($Config->useAMQPLib()) {
                $queue = $AMQP->createQueue($routing_key, false);
            } else {
                $queue = new AMQPQueue($channel);
                $queue->setName($routing_key);
                $queue->setFlags(AMQP_NOPARAM);
                $queue->declareQueue();
            }
            $Log->logNotice("Waiting on messages.....");
            printf("Waiting on messages....\n");

            init_signals();

            if (!$Config->useAMQPLib()) {
                $queue->consume(function(AMQPEnvelope $envelope, AMQPQueue $queue) use ($exchange, $manager) {
                    $returnValue = processMessage($envelope, $manager['host']);
                    if (is_array($returnValue)) {
                        $command = $returnValue['meta_command'];
                        $encoded_message = json_encode($returnValue);
                        printf("Publish response to command %s\n", $command, $envelope->getReplyTo());
                    } else {
                        printf("Error processing command, publishing error.\n", $envelope->getReplyTo());
                        $message = array('meta_command', 'error');
                        $encoded_message = json_encode($message);
                    }
                    $queue->ack($envelope->getDeliveryTag());
                    $exchange->publish($encoded_message, $envelope->getReplyTo(), AMQP_NOPARAM);
                    printf("Done, waiting on messages....\n");
                });
                $AMQP->disconnect();
            } else {
                $AMQP->consume(function($msg) use ($AMQP, $manager) {
                    global $Log;
                    global $count;
                    $Log->logInfo("Received Message " . $count++ . $msg->body . " from host " . $manager['host']);
                    $returnValue = processCommand($msg->body, $manager['host']);
                    if (is_array($returnValue)) {
                        $command = $returnValue['meta_command'];
                        $encoded_message = json_encode($returnValue);
                        printf("Publish response to command %s\n", $command);
                    } else {
                        printf("Error processing command, publishing error.\n");
                        $message = array('meta_command', 'error');
                        $encoded_message = json_encode($message);
                    }
                    $AMQP->respond($msg, $encoded_message);
                    $AMQP->ack($msg);
                    printf("Done, waiting on messages....\n");
                });
                $AMQP->waitLoop();
            }
        } catch(Exception $ex){
            $Log->logError($ex);
            $message = $ex->getMessage();
            printf("Error: " . $ex->getMessage() . "\n");
            if (strstr($message, "connection timed out")) {
                printf("connection timed out...\n");
            }
            $AMQP->disconnect();
            exit(-1);
        }
    } else {
        printf("No active management hosts are available, exiting....\n");
        $Log->logError('No active management hosts are available.');
    }
} else {
    $Log->logError('Cannot get asset tag for message routing.');
}


$count = 0;
function processMessage($envelope, $managerHost) {
    global $count;
    global $Log;
    $Log->logInfo("Received Message " . $count++ . $envelope->getBody() . " from host " . $managerHost);
    return processCommand($envelope->getBody(), $managerHost);
}

function processCommand($command, $managerHost) {
    global $vendor;
    $result = false;
    $days = 7;
    printf("Command is %s\n", $command);
    $result = $vendor->run_command($command, $days, $managerHost);
    return $result;
}

function sig_handler() {
    global $AMQP;
    $AMQP->disconnect();
    exit(-1);
}

function init_signals() {
    // setup signal handlers
    pcntl_signal(SIGTERM, "sig_handler");
    pcntl_signal(SIGHUP,  "sig_handler");
    pcntl_signal(SIGUSR1, "sig_handler");
}

?>
