<?php

class Logger {
    const DATABASE = 10;
    const DEBUG = 6;
    const HIGH = 5;
    const NORMAL = 3;
    const LOW = 1;
    const NONE = 0;
    const MONITOR_ID = "xg-portal";

    const LOG_FILE_COUNT = 4;
    const LOG_FILE_SIZE = 256;

    function __construct($logLevel = Logger::LOW, $count = Logger::LOG_FILE_COUNT, $size = Logger::LOG_FILE_SIZE, $logDirectory = "/var/log/mgportal") {
        $this->level = $logLevel;
        $this->file_count = $count;
        $this->file_size = $size;
        $this->directory = $logDirectory;
        $this->logFile = "";
        $this->baseName = "";
        $this->isOpen = false;
        $this->useHttpLog = false;
        $this->useStdout = false;

        // set things so that user and group members can write to the logs.
        umask(0002);

        /*
         * Check for invocation mode, output to screen if a "shell" by default.
         */
        if (php_sapi_name() == 'cli') {
            if (isset($_SERVER['TERM'])) {
                $invoked_by = "shell";
                $this->useStdout(true);
            } else {
                $invoked_by = "crontab";
            }
        } else {
            $invoked_by = "other";
        }
        $this->invoked_by = $invoked_by;
    }
    
    function __destruct() {
        closelog();
    }

    public function useHttpLog($useIt) {
        $this->useHttpLog = $useIt;
    }
    
    public function useStdout($useIt) {
        $this->useStdout = $useIt;
    }
    
    private function openLog() {
        if ($this->isOpen == false) {
            openlog(Logger::MONITOR_ID, LOG_NDELAY, LOG_USER);
            $this->isOpen = true;
        }
    }
    
    public function setLogLevel($logLevel) {
        $this->level = $logLevel;
    }

    public function setLogFileCount($count) {
        $this->file_count = $count;
    }

    public function setLogFileSize($size) {
        $this->file_size = $size;
    }

    public function getLogLevel() {
        return $this->level;
    }

    public function setLogFile($fileName) {
        $this->baseName = $fileName;
        $this->logFile = $this->getLogFileName();
        $this->useStdout(false);
    }

    private function getLogFileName() {
        $file = $this->baseName;
        if ($this->baseName !== "") {
            $fileName = $this->baseName;

            // 256 MB max, 4 of them.  256 * 1024 * 1000 bytes.
            $MAX_SIZE = $this->file_size * 1024 * 1000;
            $MAX_FILES = $this->file_count;

            // Find the newest file in the log file rotation and use it.
            $newestTime = 0;
            $count = 1;
            for ($i = 1; $i <= $MAX_FILES; $i++) {
                $file = $this->createLogPathname($fileName, $i);
                if (file_exists($file)) {
                    $statArray = stat($file);
                    $fileTime = $statArray['mtime'];
                    if ($newestTime == 0 || $fileTime > $newestTime) {
                        $newestTime = $fileTime;
                        $count = $i;
                    }
                } else {
                    break;
                }
            }
            // The log file name.
            $file = $this->createLogPathname($fileName, $count);

            // switch to the next file if this log file size is too large.
            if (file_exists($file) && filesize($file) > $MAX_SIZE) {
                // Go back to the first file if at the last in the rotation.
                if ($count == $MAX_FILES) {
                    $count = 1;
                } else {
                    $count++;
                }
                $file = $this->createLogPathname($fileName, $count);
                // zero out this file first.
                if (file_exists($file)) {
                    $fp = fopen($file, "w+");
                    if ($fp) {
                        fclose($fp);
                    }
                }
            }
        }
        //printf("===>File Name is " . $file . "\n");
        return $file;
    }

    private function writeToLogFile($message, $severity) {
        //printf("in WriteToLogFile " . $message . "Sev: " . $severity);
        $this->logFile = $this->getLogFileName();
        file_put_contents($this->logFile, $this->createLogMessage($message, $severity, true, true), FILE_APPEND);
    }

    private function createLogPathname($fileName, $count) {
        return $this->directory . '/' . $fileName . '_' . $count . '.log';
    }

    public function logWarning($message) {
        if ($this->useHttpLog) {
            error_log($this->createLogMessage($message, LOG_WARNING, false));
        } else if ($this->logFile === "") {
            $this->openLog();
            syslog(LOG_WARNING, $this->createLogMessage($message));
        } else {
            $this->writeToLogFile($message, " Warning"); //file_put_contents($this->logFile, $this->createLogMessage($message, " Warning", true, true), FILE_APPEND);
        }
        if ($this->useStdout) {
            printf("%s\n", $message);
        }
    }

    public function logError($message) {
        if ($this->useHttpLog) {
            error_log($this->createLogMessage($message, LOG_ERR, false));
        } else if ($this->logFile === "") {
            $this->openLog();
            syslog(LOG_ERR, $this->createLogMessage($message));
        } else {
            $this->writeToLogFile($message, " Error");
        }
        if ($this->useStdout) {
            printf("%s\n", $message);
        }
    }

    public function logNotice($message) {
        if ($this->useHttpLog) {
            error_log($this->createLogMessage($message, LOG_NOTICE, false));
        } else if ($this->logFile === "") {
            $this->openLog();
            syslog(LOG_NOTICE, $this->createLogMessage($message));
        } else {
            $this->writeToLogFile($message, " Notice");
        }
        if ($this->useStdout) {
            printf("%s\n", $message);
        }
    }

    public function logInfo($message) {
        if ($this->level >= Logger::HIGH) {
            if ($this->useHttpLog) {
                error_log($this->createLogMessage($message, LOG_INFO, false));
            } else if ($this->logFile === "") {
                $this->openLog();
                syslog(LOG_INFO, $this->createLogMessage($message));
            } else {
                $this->writeToLogFile($message, " Info");
            }
        }
        if ($this->useStdout) {
            printf("%s\n", $message);
        }
    }

    public function logDebug($message) {
        if ($this->level >= Logger::DEBUG) {
            if ($this->useHttpLog) {
                error_log($this->createLogMessage($message, LOG_INFO, false));
            } else if ($this->logFile === "") {
                $this->openLog();
                syslog(LOG_INFO, $this->createLogMessage($message));
            } else {
                $this->writeToLogFile($message, " Debug");
            }
        }
        if ($this->useStdout) {
            printf("%s\n", $message);
        }
    }

    public function createLogMessage($message, $log_type = "", $addTimestamp = false, $addnewLine = false) {
        // syslogd timestamps all messages.
        if ($addTimestamp) {
            $message = $this->getTimeString () . $log_type . ": " . $message;
        }
        if ($addnewLine) {
            $message .= "\n";
        }
        return $message;
    }
    
    private function getTimeString() {
        $timestamp = time();
        $s = date('Y/m/d H:i:s', $timestamp);
        $s .= ": ";
        return $s;
    }

    // Logs a variable with var_dump().
    function writeVariable($var, $argumentNumber = 1, $forceLog = false, $logLevel = Logger::HIGH) {
        if ($forceLog || $this->level >= $logLevel) {
            ob_start();
            var_dump($var);
            $contents = ob_get_contents();
            ob_end_clean();
            $message = "#" . $argumentNumber . ": " . $contents;
            $this->logInfo($message);
        }
    }

    /*
     * Functions for the BP class.
     *  Logs the entrance to BPL.so function, logging name, argument count and values.
     */
    public function enterFunction($stringFunction) {
        if ($this->level >= Logger::HIGH) {
            $message = $this->getTimeString() . " ENTER: " . $stringFunction;
            $nArguments = func_num_args();
            if ($nArguments == 1) {
                $message .= "(no args)\n";
            } else {
                $message .= "(" . ($nArguments - 1) . " args):\n";
            }
            $this->logInfo($message);
            for ($i = 1; $i < $nArguments; $i ++) {
                $arg = func_get_arg($i);
                $this->writeVariable($arg, $i);
            }
        }
    }

    // Logs the exit from BPL.so function, and function result value.
    public function exitFunction($stringFunction, $result) {
        if ($this->level >= Logger::HIGH) {
            $message = $this->getTimeString() . " EXIT: " . $stringFunction;
            $this->logInfo($message);
            $this->writeVariable($result);
        }
    }

}

?>