#!/usr/bin/php
<?php

const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const VENDOR_DIR = "vendor/";
const API_DIR = "/var/www/html/api/includes/";
const CONFIG_FILE = "mgportal.conf";

require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'AMQP-base.php');
require_once(BASE_DIR . BIN_DIR . 'AMQP.php');
require_once(BASE_DIR . BIN_DIR . 'REST.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');

$Log = new Logger();
// Load configuration for the client (device).
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "client");
$Config->load_configuration();
$REST = new REST($Config);
$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile('to_cloud');
// output to stdout only if called from a terminal, not from cron.
$Log->useStdout(isset($_SERVER['TERM']));
$Log->setLogFileCount($Config->getLogFileCount());
$Log->setLogFileSize($Config->getLogFileSize());

/*
 * Check to see if monitoring is turned off and exit if so.
 */
if (!$Config->anyDeviceMonitoring()) {
    $Log->logError("This device is not being managed or monitored, exiting.");
    exit(-1);
} else if ($Config->anyDeviceManagement()) {
    // check rpms
    $output = shell_exec("rpm -q librabbitmq php-pecl-amqp");
    if (strstr($output, "librabbitmq") && strstr($output, "php-pecl-amqp")) {
        // all okay, packages present.
    } else {
        $Log->logError("Management cannot be performed until dependent packages, librabbitmq and php-amqp, are installed, exiting");
        exit(-1);
    }
}

$vendor = null;
$vendorName = $Config->getVendor();
if ($vendorName !== null) {
    require_once(BASE_DIR . BIN_DIR . VENDOR_DIR . $vendorName . '.php');
    $vendor = new Vendor($REST, $Config);
}
if ($vendor === null) {
    $Log->logError("Vendor information not found; please check config file for 'vendor'");
    exit(-1);
}

$exitValue = 0;
if ($argc >= 2) {

    $options = getopt("c:d:s:h:");
    if (!isset($options["c"])) {
        usage($argv[0], $exitValue);
        exit($exitValue);
    }
    $command = $Config->get_command($options["c"]);                     // -c command
    $days = isset($options["d"]) ? $options["d"] : $Config->getDays();  // -d days or -d file (for tunnel sendkeyfile)
    $sid = isset($options["s"]) ? $options["s"] : "local";              // -s system_id or local
    $targetHost = isset($options["h"]) ? $options["h"] : "all";         // -h host

    $REST->setSID($sid);

    if ($command === Configuration::CMD_GET_ALL){
        $commands = array(Configuration::CMD_PROTECT_INFO,
                          Configuration::CMD_PROTECTED_MONTHLY,
                          Configuration::CMD_DEVICE_CHECK,
                          //Configuration::CMD_GET_JOBS, <- not using this.
                          Configuration::CMD_GET_ALERTS,
                          Configuration::CMD_GET_BACKUPS,
                          Configuration::CMD_GET_HOT_COPIES);
        // get license once
        $license = $REST->run('license');
        addName($license, $REST->getLocalID());

        $skipHosts = array();
        foreach ($commands as $command){
            $status = tryblock($vendor, $command, $REST, $Config, $Log, $days, $license, $targetHost, $skipHosts);
            if ($status === false) {
                $exitValue = -1;
                break;
            }
        }
    // original (copied above)
    } else if ($command !== false) {
        $status = tryblock($vendor, $command, $REST, $Config, $Log, $days, null, $targetHost);
        $exitValue = ($status === false) ? -1 : 0;
    } else {
        $message = "Error: " . $argv[1] . " is not a valid command";
        $Log->logError($message);
        usage($argv[0], $exitValue);
    }
    exit($exitValue);
} else {
    usage($argv[0], $exitValue);
    exit($exitValue);
}

function usage($cmd, &$exitValue) {
    printf("\n%s -c <command> [-d <# days>] [-s <system_id> | local | all] [-h <host address>]\n", $cmd);
    printf("Commands:\n\tprotect, backups, copies, alerts, device, monthly, get-connection-status, all\n\n");
    $exitValue = -1;
}

function tryblock($vendor, $command, $REST, $Config, $Log, $days, $license, $targetHost, &$skipHosts = array()){
    $success = true;
    try {
        $message = $vendor->run_command($command, $days);
        if ($message !== false) {
            if ($license === null) {
                // All messages need license information
                $license = $REST->run('license');
                addName($license, $REST->getLocalID());
            }
            $vendor->update_message($message, $license, $days);
            // All messages must have asset tag.
            if (isset($message['asset_tag']) && $message['asset_tag'] !== false) {
                $assetTag = $message['asset_tag'];
                $encoded_message = json_encode($message);
                // Good so far.  Send the message to all defined hosts.
                $hosts = $Config->getHosts();
                foreach ($hosts as $entry) {
                    $host = $entry['host'];
                    if ($targetHost != "all") {
                        if ($host != $targetHost) {
                            $Log->logInfo("Only match host " . $targetHost . ", so skipping host " . $host);
                            continue;
                        }
                    } else if (in_array($host, $skipHosts)) {
                        $Log->logInfo("Prior error seen with " . $host . ", so skipping it.");
                        continue;
                    }
                    $isManaged = $Config->isManagedByHost($host);
                    $isMonitored = $Config->isMonitoredByHost($host);
                    if ($isManaged || $isMonitored) {
                        $Log->logDebug("Message: " . ($isManaged ? "(managed): " : "(not managed): ") . $encoded_message);
                        if (!$isManaged && $command == Configuration::CMD_ADD_KEY) {
                            $Log->logError("You cannot add a key if not being managed by host $host.");
                            continue;
                        }
                        if ($isManaged && $Config->getAMQPMonitoring()) {
                            $routing_key = Configuration::DEVICE_MONITORING_ROUTE;
                            $AMQP = new AMQP($entry['host'], $Config->getUser($entry['host'], $assetTag), $assetTag);
                            $AMQP->setPort($Config->getPort());
                            $AMQP->connect($Config->useSSL());
                            $channel = $AMQP->createChannel();
                            $exchange = $AMQP->createExchange();
                            $AMQP->createQueue($routing_key);
                            if ($exchange !== null) {
                                $exchange->publish($encoded_message, $routing_key);
                            }
                            $AMQP->disconnect();
                        } else {
                            $options = "&refresh=1&devtype=";
                            $devType = mt_rand();
                            // even if managed, odd if not.
                            $options .= ($isManaged) ? $devType & ~1 : $devType | 1;
                            $urlType = ($command == Configuration::CMD_GET_KEY) ? "GET" : "POST";
                            if ($command == Configuration::CMD_GET_CONNECTION_STATUS) {
                                if ($isManaged) {
                                    $urlType = "GET";
                                    $options .= "&asset=" . $assetTag;
                                } else {
                                    $Log->logInfo("Host " . $host . " is not managing or monitoring, skip command to get connection status.");
                                    continue;
                                }
                            }
                            $portalREST = new REST($Config, false);
                            $portalREST->setAuthFromAsset($assetTag, true);
                            $result = $portalREST->run($command, $urlType, $host, $encoded_message, $options);
                            if ($portalREST->isError($result)) {
                                $Log->logError($result . " (Host: " . $host . ")");
                                $skipHosts[] = $host;
                                //$success = false;
                                // Do not exit here, but skip this host next time.
                            }
                            if ($urlType == "GET") {
                                if (is_array($result)) {
                                    if (isset($result['data'])) {
                                        $result = $result['data'];
                                    }
                                    if ($command == Configuration::CMD_GET_CONNECTION_STATUS) {
                                        if (isset($result['connection'])) {
                                            $result = "host:" . $host . ":server_connection:" . (($result['connection'] === true) ? "true" : "false");
                                        }
                                    }
                                }
                                $Log->logInfo("Result:" . $result);
                            }
                        }
                    } else {
                        $Log->logInfo("Host " . $host . " is not managing or monitoring.");
                    }
                }
            } else {
                $Log->logError("Cannot send a message with no asset tag");
                $success = false;
            }
        } else {
            $message = "Error running command " . $command;
            $Log->logError($message);
            $success = false;
        }
    } catch(Exception $ex){
        $Log->logError($ex);
    }
    return $success;
}

function addName(&$obj, $sid) {
    if (is_array($obj)) {
        $system = bp_get_system_info($sid);
        if ($system !== false) {
            $obj['hostname'] = $system['name'];
        } else {
            $obj['hostname'] = 'unknown';
        }
    }
}

?>