#!/bin/sh

VERSION=1.0.00
#set -x

#
# Create and exchange ssh RSA keys between client and server
#
# Create a reverse ssh tunnel between the client and server


#
# Only the mgportal user may run this command
#
RESULT=`/usr/bin/whoami`
if [ ! "mgportal" = "$RESULT" ]; then
    echo "Only the mgportal user can run this script."
    exit 2;
fi

# ssh standard port - this can be used or an alternate if specified
SSH_STANDARD_PORT=22

# Keys will be in mgportal $HOME directory.
KEYDIR=$HOME/.ssh
if [ ! -d $KEYDIR ]; then
    mkdir $KEYDIR
fi
BINDIR=/usr/local/mgportal/bin
KEYFILE="${HOSTNAME}_rsa"
PORTALDIR=$HOME/tunnel
if [ ! -d $PORTALDIR ]; then
    mkdir $PORTALDIR
fi
LOGFILE="$PORTALDIR/portal_log"
EXPIRE_FILE="$PORTALDIR/tunnel.exp"

CONFIG_FILE=/usr/local/mgportal/conf.d/mgportal.conf

#
#	Build ssh keys on the local appliance (client).
#   Send the public key to the manager appliance(s) to be installed as an authorized key
#
generateKeys() {
	rm -f $KEYDIR/$KEYFILE $KEYDIR/$KEYFILE.pub
	ssh-keygen -t rsa -q -f $KEYDIR/$KEYFILE -N ""
	if [ ! -f $KEYDIR/$KEYFILE.pub ]; then
	    echo "Error, no public keyfile $KEYDIR/$KEYFILE was found"
	    exit 3
	fi
    curlOutput=`$BINDIR/to_cloud.php -c addkey -d $KEYDIR/$KEYFILE.pub 2>/dev/null`
    # no error seen on this side.  Check log on server.
	exit 0
}

#
# Gets the public key from the manager and copies it to the local ssh directory.
# It will be used in creating the known_hosts value when creating a tunnel.
#
getManagerKey() {
    if [ "$1" = "" ]; then
        echo "You must provide a manager host name or address"
        exit 3
    fi
    manager_host=$1
    # use port number if provided
    if [ "$2" != "" ]; then
        portnumber=$2
    else
        portnumber=$SSH_STANDARD_PORT
    fi

    validateIP $manager_host
    if [ $? -eq 0 ]; then
    	MANAGER_NAME=`grep -w -i $manager_host /etc/hosts | awk '{print $2}' | sed -e "s/-nonVPN$//"`
    	if [ -z $MANAGER_NAME ]; then
	    MANAGER_NAME=$manager_host
	fi
    else
	MANAGER_NAME=$manager_host
    fi
    MANAGER_RSA_FILE=${MANAGER_NAME}_rsa.pub
    if [ -z $MANAGER_NAME ]; then
	    echo "Manager not found for $manager_host, error";
	    exit 3
    fi

    # get key from the manager.
    curlOutput=`$BINDIR/to_cloud.php -c getkey -h $manager_host 2>/dev/null | grep "^Result:"`
    managerKey=`echo $curlOutput | awk -F: '{print $2}'`
    if [ ! -z "$managerKey" ]; then
        rm -f $KEYDIR/$MANAGER_RSA_FILE
        echo $managerKey > $KEYDIR/$MANAGER_RSA_FILE
        if [ $? -eq 0 ]; then
            echo "successfully saved manager key file $KEYDIR/$MANAGER_RSA_FILE"
        else
            echo "error saving manager key"
            exit 3
        fi
    else
        echo "Error getting manager key"
        exit 3
    fi

    exit 0
}


#
# On the server/manager side, this is called to add the contents of the key file provided to authorized_keys
#
installKey() {
    if [ "$1" = "" ]; then
        echo "You must provide a key file name"
        exit 3
    fi
    newKeyFile=$1
    if [ ! -f $newKeyFile ]; then
        echo "Key file $newKeyFile not found"
        exit 3
    fi
    echo "adding contents of $newKeyFile to authorized keys"
    cat $newKeyFile >> $KEYDIR/authorized_keys
    if [ $? -ne 0 ]; then
        echo "error adding to keys file"
        exit 3
    fi
    exit 0
}


#
#   Create reverse tunnel for management
#
#   Arguments:  $1 = supportnumber - 5 digit tunnel port - required
#               $2 = manager host name or IP address - required
#               $3 = ssh port number, optional, defaults to 22
#
tunnel() {

    supportnumber=$1
    if [ "$1" = "" ]; then
        echo "You must input a 5 digit port number";
        exit 3
    fi
    if [ "$2" = "" ]; then
        echo "You must input a host name or IP";
        exit 3
    fi
    manager_host=$2
    # use port number if provided
    if [ "$3" != "" ]; then
        portnumber=$3
    else
        portnumber=$SSH_STANDARD_PORT
    fi

    if [ -z "$HOME" ]; then
        HOME=`getent passwd mgportal | cut -f6 -d':'`
    fi

    if [ ! -d $KEYDIR ]; then
        mkdir $KEYDIR
    fi

    chmod 400 $KEYDIR/$KEYFILE

    rm -f $KEYDIR/known_hosts

    validateIP $manager_host
    if [ $? -eq 0 ]; then
    	MANAGER_NAME=`grep -w -i $manager_host /etc/hosts | awk '{print $2}' | sed -e "s/-nonVPN$//"`
    	if [ -z $MANAGER_NAME ]; then
	    MANAGER_NAME=$manager_host
	fi
    else
	MANAGER_NAME=$manager_host
    fi
    MANAGER_RSA_FILE=${MANAGER_NAME}_rsa.pub
    if [ -z $MANAGER_NAME ]; then
	echo "Manager not found for $manager_host, error";
	exit 3;
    fi
    if [ ! -f $KEYDIR/$MANAGER_RSA_FILE ]; then
	echo "Key file not found for $MANAGER_NAME, error";
	exit 3;
    fi

    # build the known_hosts file
    if [ "$portnumber" -ne "$SSH_STANDARD_PORT" ]; then
        echo "[$manager_host]:$portnumber" `cat $KEYDIR/$MANAGER_RSA_FILE` >> $KEYDIR/known_hosts
    else
        echo "$manager_host" `cat $KEYDIR/$MANAGER_RSA_FILE` >> $KEYDIR/known_hosts
    fi

    echo "Please wait while connecting to the management server..."
    kill -9 `ps -leaf | grep ssh | grep $supportnumber | grep mgportal | awk ' { print $4 } '` 2> /dev/null
    rm -f $LOGFILE
    ssh -p $portnumber -i $KEYDIR/$KEYFILE -f -N -n -R $supportnumber:127.0.0.1:443 \
            -o "UserKnownHostsFile $KEYDIR/known_hosts"                \
            mgportal@$manager_host > $LOGFILE 2>&1

    sleep 5
    echo "Creating tunnel: $1:"
    failed=0
    if [ -s $LOGFILE ]; then
        grep -q "Warning: remote port forwarding failed" $LOGFILE
        if [[ $? == 0 ]]; then
            # This port is already in use.
            failed=1
            echo "Connection failed. The port number $supportnumber has been used. Please re-run the command to use a new port number."
        else
            ps -leaf | grep ssh | grep mgportal | grep -v grep > /dev/null 2>&1
            if [[ $? == 0 ]]; then
                # Make sure we have a tunnel
                tunnelPort=`ps -leaf | grep ssh | grep mgportal | grep -v grep | sed "s/.*-n -R //" | cut -f1 -d':'`
                #if [[ $tunnelPort != $supportnumber ]]; then
                #   failed=1
                #fi
            else
                failed=1
            fi
            if [[ $failed != 0 ]]; then
                echo -n "Connection failed. Please verify your network connection and retry.  If you continue seeing this error, or if you have changed managers, "
                echo " you may need to clean up your tunnel security keys.  Please see the Distributed Enterprise Manager KB for more information."
            fi
        fi
    fi

    if [[ $failed == 0 ]]; then
        timeStamp=`date '+%s'`
        portDays=`/usr/bp/bin/bputil -g CMC DEMPortTime 259200 /usr/bp/bpinit/master.ini`
        tunnelExp=$((timeStamp + portDays))
        echo ${tunnelExp} >$EXPIRE_FILE
        #echo "You have successfully connected to the server using port $supportnumber."
    fi
    exit 0
}

#
#   Get a local port for forwarding
#
getLocalPort() {
    if [ -x /usr/bp/bin/bputil ]; then
        portCount=`/usr/bp/bin/bputil -g CMC DEMPortCount 5 /usr/bp/bpinit/master.ini`
    elif [ -f $CONFIG_FILE ]; then
        ports=`cat $CONFIG_FILE | python -c 'import sys, json; print json.load(sys.stdin)["server"]["ports"]' 2>/dev/null`;
        if [ $? -eq 0 ]; then
            portCount=$ports
        else
            portCount=4
        fi
    fi
    usedPorts=`ps -C ssh -o args | awk 'BEGIN {used = 0} {if ($7 ~ /:localhost:/ && $10 ~ mgportal && $10 ~ known_hosts) used++} END {print used}'`
    if [ ${usedPorts} -ge ${portCount} ]; then
        localPort=''
    else
        if [ -x /usr/bp/bin/bputil ]; then
            portStart=`/usr/bp/bin/bputil -g CMC DEMPortStart 11000 /usr/bp/bpinit/master.ini`
        elif [ -f $CONFIG_FILE ]; then
            start=`cat $CONFIG_FILE | python -c 'import sys, json; print json.load(sys.stdin)["server"]["port_start"]' 2>/dev/null`;
            if [ $? -eq 0 ]; then
                portStart=$start
            else
                portStart=11005
            fi
        fi
        localPort=`expr $portStart + $usedPorts`
    fi
    echo $localPort
}

#
#   Create loopBack tunnel to the manager appliance
#
#   Arguments:  $1 = portnumber - 5 digit tunnel port - required
#
loopBack() {

    if [ "$1" = "" ]; then
        echo "You must input a 5 digit port number";
        exit 3
    fi
    portnumber=$1
    if [ "$2" = "" ]; then
        echo "You must input a host name or IP";
        exit 3
    fi
    manager_host=$2

    # Get next available port
    #localPort=$((portnumber + 1))
    localPort=`getLocalPort`
    if [ "$localPort" == "" ]; then
        echo "All ports on the manager are active.  Please close a session or increase the number of available ports.";
        exit 3
    fi

    MANAGER_NAME=$manager_host
    LOCAL_KEYFILE=localhost_rsa
    MANAGER_RSA_FILE=${LOCAL_KEYFILE}.pub
    SSH_RSA_FILE=/etc/ssh/ssh_host_rsa_key.pub


    if [ -z "$HOME" ]; then
        HOME=`getent passwd mgportal | cut -f6 -d':'`
    fi

    if [ ! -d $KEYDIR ]; then
        mkdir $KEYDIR
    fi

    if [ -z $MANAGER_NAME ]; then
	echo "Manager not found for $manager_host, error";
	exit 3;
    fi

    # if no localhost key exists, create it
    if [ ! -f $KEYDIR/$LOCAL_KEYFILE ]; then
    	ssh-keygen -t rsa -q -f $KEYDIR/$LOCAL_KEYFILE -N ""
    fi

    # ensure the localhost public key exists
    if [ ! -f $KEYDIR/$MANAGER_RSA_FILE ]; then
	echo "Key file not found for $MANAGER_NAME, error";
	exit 3;
    fi

    # see if the local host is in the known_hosts file
    grep "^$manager_host" $KEYDIR/known_hosts > /dev/null 2>&1
    if [[ $? != 0 ]]; then
            # Not there, add it
    	    #echo "$manager_host" `cat $MANAGER_RSA_FILE` >> $KEYDIR/known_hosts
    	    echo "$manager_host" `cat $SSH_RSA_FILE` >> $KEYDIR/known_hosts
    fi

    # see if the local host is in the authorized keys file
    grep "mgportal@${HOSTNAME}" $KEYDIR/authorized_keys > /dev/null 2>&1
    if [[ $? != 0 ]]; then
            # Not there, add it
    	    echo `cat $KEYDIR/$MANAGER_RSA_FILE` >> $KEYDIR/authorized_keys
    fi

    #kill -9 `ps -leaf | grep ssh | grep $localPort | grep mgportal | awk ' { print $4 } '` 2> /dev/null
    rm -f ${LOGFILE}_loop
    echo "$localPort"
    nohup ssh -g -N -i $KEYDIR/$LOCAL_KEYFILE -L $localPort:localhost:$portnumber \
            -o "UserKnownHostsFile $KEYDIR/known_hosts"                \
            -o "NoHostAuthenticationForLocalHost yes"                \
            mgportal@$manager_host > ${LOGFILE}_loop 2>&1 &
    exit 0
}

#
#   Close loopBack tunnel
#
#   Arguments:  $1 = portnumber - 5 digit tunnel port - required
#
loopBackClose() {

    if [ "$1" = "" ]; then
        echo "You must input a 5 digit port number";
        exit 3
    fi
    portnumber=$1
    #localPort=$((portnumber + 1))

    # kill existing tunnel
    kill `ps -u mgportal  -o pid,args | awk '{if ($8 ~ /:localhost:'"${portnumber}"'/ && $11 ~ mgportal && $11 ~ known_hosts) print $1}'` 2>/dev/null
}

#
# See if keys exist, returns 0 if they do, non-zero if not.
testkeys() {
    if [ "$1" = "" ]; then
        echo "You must provide a manager host name or address"
        exit 3
    fi
    removeKeys=0
    if [ "$2" = "removeKeys" ]; then
        removeKeys=1
    fi
    manager_host=$1
    validateIP $manager_host
    if [ $? -eq 0 ]; then
	MANAGER_NAME=`grep -w -i $manager_host /etc/hosts | awk '{print $2}' | sed -e "s/-nonVPN$//"`
        if [ "$MANAGER_NAME" = "" ]; then
	    MANAGER_NAME=$manager_host
        fi
    else
	MANAGER_NAME=$manager_host
    fi
    MANAGER_RSA_FILE=${MANAGER_NAME}_rsa.pub
    if [ ! -f $KEYDIR/$MANAGER_RSA_FILE ]; then
	echo "Manager key file $KEYDIR/$MANAGER_RSA_FILE not found"
	exit 3
    fi
    if [ ! -f $KEYDIR/$KEYFILE ]; then
	echo "Error, no keyfile $KEYDIR/$KEYFILE was found"
	exit 3
    fi
    if [ ! -f $KEYDIR/$KEYFILE.pub ]; then
	echo "Error, no public keyfile $KEYDIR/$KEYFILE.pub was found"
	exit 3
    fi

    if [ $removeKeys -eq 1 ]; then
        rm -f $KEYDIR/$MANAGER_RSA_FILE
        rm -f $KEYDIR/$KEYFILE
        rm -f $KEYDIR/$KEYFILE.pub
    fi

    exit 0

}

############################################
#
#   Usage function to display the usage
#
############################################
usage() {
    echo
    echo "Available options are:"
    echo
    echo "tunnel xxxxx <manager address>"
    echo " - xxxxx is a random port, address is the manager's ip or hostname"
    echo
    echo "test"
    echo " - checks to see if a tunnel is active"
    echo "   if active, returns tunnel number, else 0"
    echo
    echo "hostkey"
    echo " - generates ssh keys for tunnel, "
    echo "   sends public key to /tmp/keyFilexxxx to all management servers"
    echo
    echo "managerkey <manager address>"
    echo " - copies public key from manager at address to local keys directory"
    echo
    echo "authorizedkey <temp key filename>"
    echo " - copies public key file from temp file on manager"
    echo "   (sent from host) to local authorized_keys file"
    echo
}

validateIP() {
    local ip=$1
    local stat=1
    if [[ $ip =~ ^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$ ]]; then
        OIFS=$IFS
        IFS='.'
        ip=($ip)
        IFS=$OIFS
        [[ ${ip[0]} -le 255 && ${ip[1]} -le 255 \
        && ${ip[2]} -le 255 && ${ip[3]} -le 255 ]]
        stat=$?
    fi
    return $stat
}


# brought over from cmc_active_tunnel, for mgportal.
checkForTunnel() {
    #
    # KAS 7/6/2015
    # This script will look for an active support tunnel. If found then the port number is written to stdout.
    # If no active tunnel is found, then 0 is written to stdout.
    #

    ps -leaf | grep ssh | grep mgportal | grep -v grep > /dev/null 2>&1
    if [[ $? == 0 ]]; then
            # Found active legacy tunnel.
            PORT=`ps -leaf | grep ssh | grep mgportal | grep -v grep | sed "s/.*-n -R //" | cut -f1 -d':'`
            echo -n $PORT
    else
            # No active tunnel.
            echo -n 0
    fi
    exit 0
}

############################################
#
#   Usage
#
############################################
case "$1" in
'tunnel')
    tunnel $2 $3
    ;;
'loopBack')
    loopBack $2 $3
    ;;
'loopBackClose')
    loopBackClose $2 $3
    ;;
'testkeys')
    testkeys $2 $3
    ;;
'hostkey')
    generateKeys
    ;;
'managerkey')
    getManagerKey $2
    ;;
'authorizedKey')
    installKey $2
    ;;
'test')
    checkForTunnel
    ;;
*)
    echo "tunnel V. $VERSION usage: $0 [options]"
    usage
    ;;
esac
